@@ -3,10 +3,10 @@
  *
  *	  The C functions and triggers portion of Slony-I.
  *
- *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2005, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slony1_funcs.c,v 1.33.2.5 2006-05-15 14:47:31 wieck Exp $
+ *	$Id: slony1_funcs.c,v 1.53 2006/09/07 13:21:16 xfade Exp $
  * ----------------------------------------------------------------------
  */
 
@@ -22,8 +22,10 @@
 #include "commands/async.h"
 #include "catalog/pg_operator.h"
 #include "access/xact.h"
+#include "access/transam.h"
 #include "utils/builtins.h"
 #include "utils/elog.h"
+#include "utils/guc.h"
 #ifdef HAVE_TYPCACHE
 #include "utils/typcache.h"
 #else
@@ -34,6 +36,9 @@
 #include <signal.h>
 #include <errno.h>
 
+#ifdef PG_MODULE_MAGIC
+PG_MODULE_MAGIC;
+#endif
 
 PG_FUNCTION_INFO_V1(_Slony_I_createEvent);
 PG_FUNCTION_INFO_V1(_Slony_I_getLocalNodeId);
@@ -99,6 +104,7 @@ typedef struct slony_I_cluster_status
 	void	   *plan_insert_log_1;
 	void	   *plan_insert_log_2;
 	void	   *plan_record_sequences;
+	void	   *plan_get_logstatus;
 
 	text	   *cmdtype_I;
 	text	   *cmdtype_U;
@@ -115,7 +121,7 @@ static Slony_I_ClusterStatus *clusterStatusList = NULL;
 static Slony_I_ClusterStatus *
 getClusterStatus(Name cluster_name,
 				 int need_plan_mask);
-const char * slon_quote_identifier(const char *ident);
+const char *slon_quote_identifier(const char *ident);
 static char *slon_quote_literal(char *str);
 
 
@@ -126,8 +132,8 @@ _Slony_I_createEvent(PG_FUNCTION_ARGS)
 	Slony_I_ClusterStatus *cs;
 	text	   *ev_xip;
 	char	   *ev_type_c;
-	Datum		argv  [12];
-	char		nulls  [13];
+	Datum		argv[12];
+	char		nulls[13];
 	char	   *buf;
 	size_t		buf_size;
 	int			rc;
@@ -384,7 +390,7 @@ _Slony_I_logTrigger(PG_FUNCTION_ARGS)
 	TransactionId newXid = GetTopTransactionId();
 	Slony_I_ClusterStatus *cs;
 	TriggerData *tg;
-	Datum		argv  [4];
+	Datum		argv[4];
 	text	   *cmdtype = NULL;
 	int			rc;
 	Name		cluster_name;
@@ -443,7 +449,11 @@ _Slony_I_logTrigger(PG_FUNCTION_ARGS)
 			break;
 
 		case SLON_ROLE_SLON:	/* non-client session ??? */
-			elog(ERROR, "Slony-I: logTrigger() called in non-client session");
+			/* This would happen when a trigger on a
+			 * subscriber on a replicated table fires, and
+			 * modifies a tuple in a replication set for
+			 * which this node is the origin */
+			break;
 	}
 
 	/*
@@ -451,10 +461,33 @@ _Slony_I_logTrigger(PG_FUNCTION_ARGS)
 	 */
 	if (!TransactionIdEquals(cs->currentXid, newXid))
 	{
+		int32	log_status;
+
 		/*
 		 * Determine the currently active log table
 		 */
-		cs->plan_active_log = cs->plan_insert_log_1;
+		if(SPI_execp(cs->plan_get_logstatus, NULL, NULL, 0) < 0)
+			elog(ERROR, "Slony-I: cannot determine log status");
+		if (SPI_processed != 1)
+			elog(ERROR, "Slony-I: cannot determine log status");
+		
+		log_status = DatumGetInt32(SPI_getbinval(SPI_tuptable->vals[0],
+									SPI_tuptable->tupdesc, 1, NULL));
+		SPI_freetuptable(SPI_tuptable);
+
+		switch(log_status)
+		{
+			case 0:
+			case 2:		cs->plan_active_log = cs->plan_insert_log_1;
+						break;
+			
+			case 1:
+			case 3:		cs->plan_active_log = cs->plan_insert_log_2;
+						break;
+
+			default:	elog(ERROR, "Slony-I: illegal log status %d", log_status);
+						break;
+		}
 
 		cs->currentXid = newXid;
 	}
@@ -467,31 +500,22 @@ _Slony_I_logTrigger(PG_FUNCTION_ARGS)
 		HeapTuple	new_row = tg->tg_trigtuple;
 		TupleDesc	tupdesc = tg->tg_relation->rd_att;
 		char	   *col_ident;
-		char	  **col_value;
+		char	   *col_value;
+		
 		int			len_ident;
 		int			len_value;
 		int			i;
 		int			need_comma = false;
+		char			*OldDateStyle;
 		char	   *cp = VARDATA(cs->cmddata_buf);
 
 		/*
 		 * INSERT
 		 *
-		 * cmdtype = 'I' cmddata = ("non-NULL-col" [, ...]) values ('value' [,
-		 * ...])
+		 * cmdtype = 'I' cmddata = ("col" [, ...]) values ('value' [, ...])
 		 */
 		cmdtype = cs->cmdtype_I;
 
-		/*
-		 * Allocate an array of char pointers to hold the values. We need to
-		 * go over the tuple descriptor 2 times, first to add the column names
-		 * of non-null columns, second to add the values. But we can identify
-		 * what's NULL only by getting the value via SPI_getvalue() in the
-		 * first pass.
-		 */
-		col_value = (char **)palloc(sizeof(char *) *
-									tg->tg_relation->rd_att->natts);
-
 		/*
 		 * Specify all the columns
 		 */
@@ -499,16 +523,12 @@ _Slony_I_logTrigger(PG_FUNCTION_ARGS)
 		for (i = 0; i < tg->tg_relation->rd_att->natts; i++)
 		{
 			/*
-			 * Skip dropped columns and NULL values
+			 * Skip dropped columns
 			 */
 			if (tupdesc->attrs[i]->attisdropped)
 				continue;
-			if ((col_value[i] = SPI_getvalue(new_row, tupdesc, i + 1)) == NULL)
-				continue;
 
 			col_ident = (char *)slon_quote_identifier(SPI_fname(tupdesc, i + 1));
-			col_value[i] = slon_quote_literal(col_value[i]);
-
 			cmddata_need = (cp - (char *)(cs->cmddata_buf)) + 16 +
 				(len_ident = strlen(col_ident));
 			if (cs->cmddata_size < cmddata_need)
@@ -548,18 +568,28 @@ _Slony_I_logTrigger(PG_FUNCTION_ARGS)
 		 * Append the values
 		 */
 		need_comma = false;
+		OldDateStyle=GetConfigOptionByName("DateStyle", NULL);
+		set_config_option("DateStyle", "ISO", PGC_USERSET, PGC_S_SESSION, true, true);
 		for (i = 0; i < tg->tg_relation->rd_att->natts; i++)
 		{
 			/*
-			 * Skip dropped columns and NULL values
+			 * Skip dropped columns
 			 */
 			if (tupdesc->attrs[i]->attisdropped)
 				continue;
-			if (col_value[i] == NULL)
-				continue;
+
+
+			if ((col_value = SPI_getvalue(new_row, tupdesc, i + 1)) == NULL)
+			{
+				col_value = "NULL";
+			}
+			else
+			{
+				col_value = slon_quote_literal(col_value);
+			}
 
 			cmddata_need = (cp - (char *)(cs->cmddata_buf)) + 16 +
-				(len_value = strlen(col_value[i]));
+				(len_value = strlen(col_value));
 			if (cs->cmddata_size < cmddata_need)
 			{
 				int			have = (cp - (char *)(cs->cmddata_buf));
@@ -575,10 +605,12 @@ _Slony_I_logTrigger(PG_FUNCTION_ARGS)
 			else
 				need_comma = true;
 
-			memcpy(cp, col_value[i], len_value);
+			memcpy(cp, col_value, len_value);
 			cp += len_value;
 		}
 
+		set_config_option("DateStyle", OldDateStyle, PGC_USERSET, PGC_S_SESSION, true, true);
+
 		/*
 		 * Terminate and done
 		 */
@@ -604,6 +636,8 @@ _Slony_I_logTrigger(PG_FUNCTION_ARGS)
 		int			i;
 		int			need_comma = false;
 		int			need_and = false;
+		char		*OldDateStyle;
+
 		char	   *cp = VARDATA(cs->cmddata_buf);
 
 		/*
@@ -697,8 +731,12 @@ _Slony_I_logTrigger(PG_FUNCTION_ARGS)
 			if (new_isnull)
 				col_value = "NULL";
 			else
+			{
+				OldDateStyle=GetConfigOptionByName("DateStyle", NULL);
+				set_config_option("DateStyle", "ISO", PGC_USERSET, PGC_S_SESSION, true, true);
 				col_value = slon_quote_literal(SPI_getvalue(new_row, tupdesc, i + 1));
-
+				set_config_option("DateStyle", OldDateStyle, PGC_USERSET, PGC_S_SESSION, true, true);
+			}
 			cmddata_need = (cp - (char *)(cs->cmddata_buf)) + 16 +
 				(len_ident = strlen(col_ident)) +
 				(len_value = strlen(col_value));
@@ -1013,8 +1051,8 @@ _Slony_I_killBackend(PG_FUNCTION_ARGS)
 	if (!superuser())
 		elog(ERROR, "Slony-I: insufficient privilege for killBackend");
 
-	pid		= PG_GETARG_INT32(0);
-	signame	= PG_GETARG_TEXT_P(1);
+	pid = PG_GETARG_INT32(0);
+	signame = PG_GETARG_TEXT_P(1);
 
 	if (VARSIZE(signame) == VARHDRSZ + 4 &&
 		memcmp(VARDATA(signame), "NULL", 0) == 0)
@@ -1022,12 +1060,13 @@ _Slony_I_killBackend(PG_FUNCTION_ARGS)
 		signo = 0;
 	}
 	else if (VARSIZE(signame) == VARHDRSZ + 4 &&
-		memcmp(VARDATA(signame), "TERM", 0) == 0)
+			 memcmp(VARDATA(signame), "TERM", 0) == 0)
 	{
 		signo = SIGTERM;
 	}
 	else
 	{
+		signo = 0;
 		elog(ERROR, "Slony-I: unsupported signal");
 	}
 
@@ -1083,7 +1122,7 @@ slon_quote_literal(char *str)
 
 
 /*
- * slon_quote_identifier                     - Quote an identifier only if needed
+ * slon_quote_identifier					 - Quote an identifier only if needed
  *
  * When quotes are needed, we palloc the required space; slightly
  * space-wasteful but well worth it for notational simplicity.
@@ -1093,109 +1132,108 @@ slon_quote_literal(char *str)
 const char *
 slon_quote_identifier(const char *ident)
 {
-        /*
-         * Can avoid quoting if ident starts with a lowercase letter or
-         * underscore and contains only lowercase letters, digits, and
-         * underscores, *and* is not any SQL keyword.  Otherwise, supply
-         * quotes.
-         */
-        int                     nquotes = 0;
-        bool            safe;
-        const char *ptr;
-        char       *result;
-        char       *optr;
+	/*
+	 * Can avoid quoting if ident starts with a lowercase letter or underscore
+	 * and contains only lowercase letters, digits, and underscores, *and* is
+	 * not any SQL keyword.  Otherwise, supply quotes.
+	 */
+	int			nquotes = 0;
+	bool		safe;
+	const char *ptr;
+	char	   *result;
+	char	   *optr;
 
-        /*
-         * would like to use <ctype.h> macros here, but they might yield
-         * unwanted locale-specific results...
-         */
-        safe = ((ident[0] >= 'a' && ident[0] <= 'z') || ident[0] == '_');
+	/*
+	 * would like to use <ctype.h> macros here, but they might yield unwanted
+	 * locale-specific results...
+	 */
+	safe = ((ident[0] >= 'a' && ident[0] <= 'z') || ident[0] == '_');
 
-        for (ptr = ident; *ptr; ptr++)
-        {
-                char            ch = *ptr;
+	for (ptr = ident; *ptr; ptr++)
+	{
+		char		ch = *ptr;
 
-                if ((ch >= 'a' && ch <= 'z') ||
-                        (ch >= '0' && ch <= '9') ||
-                        (ch == '_'))
-                {
-                        /* okay */
-                }
-                else
-                {
-                        safe = false;
-                        if (ch == '"')
-                                nquotes++;
-                }
-        }
+		if ((ch >= 'a' && ch <= 'z') ||
+			(ch >= '0' && ch <= '9') ||
+			(ch == '_'))
+		{
+			/* okay */
+		}
+		else
+		{
+			safe = false;
+			if (ch == '"')
+				nquotes++;
+		}
+	}
 
-        if (safe)
-        {
-                /*
-                 * Check for keyword.  This test is overly strong, since many of
-                 * the "keywords" known to the parser are usable as column names,
-                 * but the parser doesn't provide any easy way to test for whether
-                 * an identifier is safe or not... so be safe not sorry.
-                 *
-                 * Note: ScanKeywordLookup() does case-insensitive comparison, but
-                 * that's fine, since we already know we have all-lower-case.
-                 */
-                if (ScanKeywordLookup(ident) != NULL)
-                        safe = false;
-        }
+	if (safe)
+	{
+		/*
+		 * Check for keyword.  This test is overly strong, since many of the
+		 * "keywords" known to the parser are usable as column names, but the
+		 * parser doesn't provide any easy way to test for whether an
+		 * identifier is safe or not... so be safe not sorry.
+		 *
+		 * Note: ScanKeywordLookup() does case-insensitive comparison, but that's
+		 * fine, since we already know we have all-lower-case.
+		 */
+		if (ScanKeywordLookup(ident) != NULL)
+			safe = false;
+	}
 
-        if (safe)
-                return ident;                   /* no change needed */
+	if (safe)
+		return ident;			/* no change needed */
 
-        result = (char *) palloc(strlen(ident) + nquotes + 2 + 1);
+	result = (char *)palloc(strlen(ident) + nquotes + 2 + 1);
 
-        optr = result;
-        *optr++ = '"';
-        for (ptr = ident; *ptr; ptr++)
-        {
-                char            ch = *ptr;
+	optr = result;
+	*optr++ = '"';
+	for (ptr = ident; *ptr; ptr++)
+	{
+		char		ch = *ptr;
 
-                if (ch == '"')
-                        *optr++ = '"';
-                *optr++ = ch;
-        }
-        *optr++ = '"';
-        *optr = '\0';
+		if (ch == '"')
+			*optr++ = '"';
+		*optr++ = ch;
+	}
+	*optr++ = '"';
+	*optr = '\0';
 
-        return result;
+	return result;
 }
 
 
 
 /*
  * _slon_quote_ident -
- *        returns a properly quoted identifier
+ *		  returns a properly quoted identifier
  *
  * Version: pgsql/src/backend/utils/adt/quote.c,v 1.14.4.1 2005/03/21 16:29:31
  */
 Datum
 _slon_quote_ident(PG_FUNCTION_ARGS)
 {
-        text       *t = PG_GETARG_TEXT_P(0);
-        text       *result;
-        const char *qstr;
-        char       *str;
-        int                     len;
+	text	   *t = PG_GETARG_TEXT_P(0);
+	text	   *result;
+	const char *qstr;
+	char	   *str;
+	int			len;
 
-        /* We have to convert to a C string to use quote_identifier */
-        len = VARSIZE(t) - VARHDRSZ;
-        str = (char *) palloc(len + 1);
-        memcpy(str, VARDATA(t), len);
-        str[len] = '\0';
+	/* We have to convert to a C string to use quote_identifier */
+	len = VARSIZE(t) - VARHDRSZ;
+	str = (char *)palloc(len + 1);
+	memcpy(str, VARDATA(t), len);
+	str[len] = '\0';
 
-        qstr = slon_quote_identifier(str);
+	qstr = slon_quote_identifier(str);
 
-        len = strlen(qstr);
-        result = (text *) palloc(len + VARHDRSZ);
-        VARATT_SIZEP(result) = len + VARHDRSZ;
-        memcpy(VARDATA(result), qstr, len);
+	len = strlen(qstr);
+	result = (text *) palloc(len + VARHDRSZ);
+	VARATT_SIZEP(result) = len + VARHDRSZ;
+	memcpy(VARDATA(result), qstr, len);
 
-        PG_RETURN_TEXT_P(result);
+	PG_RETURN_TEXT_P(result);
 }
 
 
@@ -1205,7 +1243,7 @@ getClusterStatus(Name cluster_name, int need_plan_mask)
 {
 	Slony_I_ClusterStatus *cs;
 	int			rc;
-	char		query  [1024];
+	char		query[1024];
 	bool		isnull;
 	Oid			plan_types[12];
 	Oid			xxid_typid;
@@ -1265,7 +1303,7 @@ getClusterStatus(Name cluster_name, int need_plan_mask)
 										SPI_getbinval(SPI_tuptable->vals[0], SPI_tuptable->tupdesc, 1, &isnull));
 		SPI_freetuptable(SPI_tuptable);
 		if (cs->localNodeId < 0)
-			elog(ERROR, "Slony-I: Node is uninitialized");
+			elog(ERROR, "Slony-I: Node is uninitialized - cluster %s", DatumGetCString(cluster_name));
 
 		/*
 		 * Initialize the currentXid to invalid
@@ -1280,7 +1318,7 @@ getClusterStatus(Name cluster_name, int need_plan_mask)
 	}
 
 	/*
-	 * Prepare and save the PLAN_NOTIFT_EVENT
+	 * Prepare and save the PLAN_NOTIFY_EVENT
 	 */
 	if ((need_plan_mask & PLAN_NOTIFY_EVENT) != 0 &&
 		(cs->have_plan & PLAN_NOTIFY_EVENT) == 0)
@@ -1306,8 +1344,12 @@ getClusterStatus(Name cluster_name, int need_plan_mask)
 		xxid_typename->names =
 			lappend(lappend(NIL, makeString(NameStr(cs->clustername))),
 					makeString("xxid"));
-		xxid_typid = typenameTypeId(xxid_typename);
 
+#ifdef HAVE_TYPENAMETYPEID_2
+		xxid_typid = typenameTypeId(NULL,xxid_typename);
+#else
+		xxid_typid = typenameTypeId(xxid_typename);
+#endif
 		plan_types[0] = INT4OID;
 
 		/*
@@ -1386,7 +1428,11 @@ getClusterStatus(Name cluster_name, int need_plan_mask)
 		xxid_typename->names =
 			lappend(lappend(NIL, makeString(NameStr(cs->clustername))),
 					makeString("xxid"));
+#ifdef HAVE_TYPENAMETYPEID_2
+		xxid_typid = typenameTypeId(NULL, xxid_typename);
+#else
 		xxid_typid = typenameTypeId(xxid_typename);
+#endif
 
 		/*
 		 * Create the saved plan's
@@ -1435,6 +1481,13 @@ getClusterStatus(Name cluster_name, int need_plan_mask)
 		VARATT_SIZEP(cs->cmdtype_D) = VARHDRSZ + 1;
 		*VARDATA(cs->cmdtype_D) = 'D';
 
+		/*
+		 * And the plan to read the current log_status.
+		 */
+		sprintf(query, "SELECT last_value::int4 FROM %s.sl_log_status",
+				cs->clusterident);
+		cs->plan_get_logstatus = SPI_saveplan(SPI_prepare(query, 0, NULL));
+
 		cs->cmddata_size = 8192;
 		cs->cmddata_buf = (text *) malloc(8192);
 
