@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slon.c,v 1.66 2006/08/02 14:29:39 xfade Exp $
+ *	$Id: slon.c,v 1.66.2.5 2006/12/13 14:39:16 darcyb Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -31,11 +31,12 @@
 
 #include "libpq-fe.h"
 #include "c.h"
-
+ 
 #include "slon.h"
 #include "confoptions.h"
 
 
+
 /* ---------- 
  * Global data 
  * ----------
@@ -326,7 +327,7 @@ main(int argc, char *const argv[])
 		}
 		else
 		{
-			slon_log(SLON_FATAL, "Cannot open pid_file \"%s\", pid_file\n");
+			slon_log(SLON_FATAL, "Cannot open pid_file \"%s\"\n", pid_file);
 			exit(-1);
 		}
 	}
@@ -391,15 +392,17 @@ SlonMain(void)
 	startup_conn = PQconnectdb(rtcfg_conninfo);
 	if (startup_conn == NULL)
 	{
-		slon_log(SLON_FATAL, "main: PQconnectdb() failed\n");
+		slon_log(SLON_FATAL, "main: PQconnectdb() failed - sleep 10s\n");
+		sleep (10);
 		slon_retry();
 		exit(-1);
 	}
 	if (PQstatus(startup_conn) != CONNECTION_OK)
 	{
-		slon_log(SLON_FATAL, "main: Cannot connect to local database - %s\n",
+		slon_log(SLON_FATAL, "main: Cannot connect to local database - %s - sleep 10s\n",
 				 PQerrorMessage(startup_conn));
 		PQfinish(startup_conn);
+		sleep(10);
 		slon_retry();
 		exit(-1);
 	}
@@ -410,7 +413,8 @@ SlonMain(void)
 	rtcfg_nodeid = db_getLocalNodeId(startup_conn);
 	if (rtcfg_nodeid < 0)
 	{
-		slon_log(SLON_FATAL, "main: Node is not initialized properly\n");
+		slon_log(SLON_FATAL, "main: Node is not initialized properly - sleep 10s\n");
+		sleep(10);
 		slon_retry();
 		exit(-1);
 	}
@@ -468,8 +472,9 @@ SlonMain(void)
 				 "set transaction isolation level serializable;");
 	if (PQresultStatus(res) != PGRES_COMMAND_OK)
 	{
-		slon_log(SLON_FATAL, "Cannot start transaction - %s\n",
+		slon_log(SLON_FATAL, "Cannot start transaction - %s - sleep 10s\n",
 				 PQresultErrorMessage(res));
+		sleep (10);
 		PQclear(res);
 		slon_retry();
 	}
@@ -860,8 +865,15 @@ SlonWatchdog(void)
 		case SLON_WATCHDOG_NORMAL:
 		case SLON_WATCHDOG_RETRY:
 			watchdog_status = SLON_WATCHDOG_RETRY;
-			slon_log(SLON_DEBUG1, "slon: restart of worker in 10 seconds\n");
-			sleep(10);
+			if (child_status != 0)
+			{
+				slon_log(SLON_DEBUG1, "slon: restart of worker in 10 seconds\n");
+				sleep(10);
+			}
+			else
+			{
+				slon_log(SLON_DEBUG1, "slon: restart of worker\n");
+			}
 			if (watchdog_status == SLON_WATCHDOG_RETRY)
 			{
 				execvp(main_argv[0], main_argv);
@@ -943,7 +955,8 @@ slon_terminate_worker()
 		kill(slon_worker_pid, SIGKILL);
 		slon_exit(-1);
 	}
-
+        close(sched_wakeuppipe[0]);
+	close(sched_wakeuppipe[1]);
 	alarm(20);
 }
 #endif
