@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slonik.c,v 1.67 2006/08/04 20:31:27 xfade Exp $
+ *	$Id: slonik.c,v 1.67.2.3 2006/12/12 14:52:59 xfade Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -70,51 +70,8 @@ static void	script_commit_all(SlonikStmt * stmt,
 static void script_rollback_all(SlonikStmt * stmt,
 					SlonikScript * script);
 static void script_disconnect_all(SlonikScript * script);
-
-/*
- * make a copy of the array of lines, with token replaced by replacement
- * the first time it occurs on each line.
- *
- * This does most of what sed was used for in the shell script, but
- * doesn't need any regexp stuff.
- */
-void
-replace_token(char *resout, char *lines, const char *token, const char *replacement)
-{
-	int			numlines = 1;
-	int			i,
-				o;
-	char		result_set[4096];
-	int			toklen,
-				replen;
-
-	for (i = 0; lines[i]; i++)
-		numlines++;
-
-	toklen = strlen(token);
-	replen = strlen(replacement);
-
-	for (i = o = 0; i < numlines; i++, o++)
-	{
-		/* just copy pointer if NULL or no change needed */
-		if (!lines[i] || (strncmp((const char *)lines + i, token, toklen)))
-		{
-			if (lines[i] == 0x0d)		/* ||(lines[i] == 0x0a)) */
-				break;
-
-			result_set[o] = lines[i];
-			continue;
-		}
-		/* if we get here a change is needed - set up new line */
-		strncpy((char *)result_set + o, replacement, replen);
-		o += replen - 1;
-		i += toklen - 1;
-	}
-
-	result_set[o] = '\0';
-	memcpy(resout, result_set, o);
-
-}
+static void replace_token(char *resout, char *lines, const char *token, 
+					const char *replacement);
 
 /* ----------
  * main
@@ -1139,6 +1096,41 @@ script_check_stmts(SlonikScript * script, SlonikStmt * hdr)
 				}
 				break;
 
+			case STMT_SYNC:
+				{
+					SlonikStmt_sync *stmt =
+					(SlonikStmt_sync *) hdr;
+
+					if (stmt->no_id == -1)
+					{
+						printf("%s:%d: Error: "
+							   "node ID must be specified\n",
+							   hdr->stmt_filename, hdr->stmt_lno);
+						errors++;
+					}
+
+					if (script_check_adminfo(hdr, stmt->no_id) < 0)
+						errors++;
+
+				}
+				break;
+			case STMT_SLEEP:
+				{
+					SlonikStmt_sleep *stmt =
+					(SlonikStmt_sleep *) hdr;
+
+					if (stmt->num_secs < 0)
+					{
+						printf("%s:%d: Error: "
+							   " sleep time (%d) must be positive\n",
+						       hdr->stmt_filename, hdr->stmt_lno, stmt->num_secs);
+						errors++;
+					}
+
+
+				}
+				break;
+
 		}
 
 		hdr = hdr->next;
@@ -1574,6 +1566,25 @@ script_exec_stmts(SlonikScript * script, SlonikStmt * hdr)
 				}
 				break;
 
+			case STMT_SYNC:
+				{
+					SlonikStmt_sync *stmt =
+					(SlonikStmt_sync *) hdr;
+
+					if (slonik_sync(stmt) < 0)
+						errors++;
+				}
+				break;
+			case STMT_SLEEP:
+				{
+					SlonikStmt_sleep *stmt =
+					(SlonikStmt_sleep *) hdr;
+
+					if (slonik_sleep(stmt) < 0)
+						errors++;
+				}
+				break;
+
 		}
 
 		if (current_try_level == 0)
@@ -1871,12 +1882,12 @@ load_slony_base(SlonikStmt * stmt, int no_id)
 		use_major = 7;
 		use_minor = 4;
 	}
-	else if ((adminfo->pg_version >= 70500) && adminfo->pg_version < 80200)	/* 8.0 and 8.1 */ 
+	else if ((adminfo->pg_version >= 70500) && adminfo->pg_version < 80300)	/* 8.0, 8.1 and 8.2 */ 
 	{
 		use_major = 8;
 		use_minor = 0;
 	}
-	else	/* 8.2 and above */
+	else	/* 8.3 and above */
 	{
 		use_major = 8;
 		use_minor = 0;
@@ -4186,6 +4197,43 @@ slonik_switch_log(SlonikStmt_switch_log * stmt)
 }
 
 
+int
+slonik_sync(SlonikStmt_sync * stmt)
+{
+	SlonikAdmInfo *adminfo1;
+	SlonDString query;
+
+	adminfo1 = get_active_adminfo((SlonikStmt *) stmt, stmt->no_id);
+	if (adminfo1 == NULL)
+		return -1;
+
+	if (db_begin_xact((SlonikStmt *) stmt, adminfo1) < 0)
+		return -1;
+
+	dstring_init(&query);
+
+	slon_mkquery(&query,
+				 "select \"_%s\".createEvent('_%s', 'SYNC'); ",
+				 stmt->hdr.script->clustername,
+				 stmt->hdr.script->clustername);
+	if (db_exec_command((SlonikStmt *) stmt, adminfo1, &query) < 0)
+	{
+		dstring_free(&query);
+		return -1;
+	}
+
+	dstring_free(&query);
+	return 0;
+}
+
+int
+slonik_sleep(SlonikStmt_sleep * stmt)
+{
+	sleep(stmt->num_secs);
+	return 0;
+}
+
+
 /*
  * scanint8 --- try to parse a string into an int8.
  *
@@ -4252,6 +4300,52 @@ slon_scanint64(char *str, int64 * result)
 	return true;
 }
 
+
+/*
+ * make a copy of the array of lines, with token replaced by replacement
+ * the first time it occurs on each line.
+ *
+ * This does most of what sed was used for in the shell script, but
+ * doesn't need any regexp stuff.
+ */
+static void
+replace_token(char *resout, char *lines, const char *token, const char *replacement)
+{
+	int			numlines = 1;
+	int			i,
+				o;
+	char		result_set[4096];
+	int			toklen,
+				replen;
+
+	for (i = 0; lines[i]; i++)
+		numlines++;
+
+	toklen = strlen(token);
+	replen = strlen(replacement);
+
+	for (i = o = 0; i < numlines; i++, o++)
+	{
+		/* just copy pointer if NULL or no change needed */
+		if (!lines[i] || (strncmp((const char *)lines + i, token, toklen)))
+		{
+			if (lines[i] == 0x0d)		/* ||(lines[i] == 0x0a)) */
+				break;
+
+			result_set[o] = lines[i];
+			continue;
+		}
+		/* if we get here a change is needed - set up new line */
+		strncpy((char *)result_set + o, replacement, replen);
+		o += replen - 1;
+		i += toklen - 1;
+	}
+
+	result_set[o] = '\0';
+	memcpy(resout, result_set, o);
+}
+
+
 /*
  * Local Variables:
  *	tab-width: 4
