@@ -1,5 +1,5 @@
 # -*- perl -*-
-# $Id: slon-tools.pm,v 1.26 2005/11/15 18:09:59 cbbrowne Exp $
+# $Id: slon-tools.pm,v 1.26.2.2 2006/12/06 18:38:41 cbbrowne Exp $
 # Author: Christopher Browne
 # Copyright 2004 Afilias Canada
 
@@ -83,18 +83,15 @@ sub genheader {
 # then invokes it and deletes it
 sub run_slonik_script {
   my ($script) = @_;
-  chomp $script;
   open(OUT, ">>$LOGDIR/slonik_scripts.log");
   my $now = `date`;
   chomp $now;
   print OUT "# -------------------------------------------------------------\n";
   print OUT "# Script: $script submitted at $now \n";
   print OUT "# -------------------------------------------------------------\n";
+  print OUT $script;
   close OUT;
-  `cat $script >> $LOGDIR/slonik_scripts.log`;
-  #print `slonik < $script`;
-  print `cat $script`;
-  unlink($script);
+  print $script;
 }
 
 sub ps_args {
@@ -136,7 +133,7 @@ sub start_slon {
   my ($dsn, $dbname) = ($DSN[$nodenum], $DBNAME[$nodenum]);
   $SYNC_CHECK_INTERVAL ||= 1000;
   system("mkdir -p $LOGDIR/slony1/node$nodenum");
-  my $cmd = "@@PGBINDIR@@/slon -s $SYNC_CHECK_INTERVAL -d2 $CLUSTER_NAME '$dsn' 2>&1 ";
+  my $cmd = "@@SLONBINDIR@@/slon -s $SYNC_CHECK_INTERVAL -d2 $CLUSTER_NAME '$dsn' 2>&1 ";
   if ($APACHE_ROTATOR) {
     $cmd .= "| $APACHE_ROTATOR \"$LOGDIR/slony1/node$nodenum/" .  $dbname . "_%Y-%m-%d_%H:%M:%S.log\" 10M &";
   } else {
